@@ -1,5 +1,5 @@
 /*
- * $Id: avrcore.c,v 1.81 2005/01/13 20:27:59 zfrdh Exp $
+ * $Id: avrcore.c,v 1.86 2008/01/07 14:26:25 joerg_wunsch Exp $
  *
  ****************************************************************************
  *
@@ -81,11 +81,11 @@ struct _BreakPt
 
 #endif
 
-static inline BreakPt *brk_pt_new (int pc, uint16_t opcode);
-static inline void brk_pt_construct (BreakPt *bp, int pc, uint16_t opcode);
-static inline void brk_pt_destroy (void *bp);
+static BreakPt *brk_pt_new (int pc, uint16_t opcode);
+static void brk_pt_construct (BreakPt *bp, int pc, uint16_t opcode);
+static void brk_pt_destroy (void *bp);
 
-static inline BreakPt *
+static BreakPt *
 brk_pt_new (int pc, uint16_t opcode)
 {
     BreakPt *bp;
@@ -97,7 +97,7 @@ brk_pt_new (int pc, uint16_t opcode)
     return bp;
 }
 
-static inline void
+static void
 brk_pt_construct (BreakPt *bp, int pc, uint16_t opcode)
 {
     if (bp == NULL)
@@ -109,7 +109,7 @@ brk_pt_construct (BreakPt *bp, int pc, uint16_t opcode)
     bp->opcode = opcode;
 }
 
-static inline void
+static void
 brk_pt_destroy (void *bp)
 {
     BreakPt *_bp = (BreakPt *)bp;
@@ -120,9 +120,9 @@ brk_pt_destroy (void *bp)
     class_destroy (bp);
 }
 
-static inline DList *brk_pt_list_add (DList *head, int pc, uint16_t opcode);
-static inline DList *brk_pt_list_delete (DList *head, int pc);
-static inline BreakPt *brk_pt_list_lookup (DList *head, int pc);
+static DList *brk_pt_list_add (DList *head, int pc, uint16_t opcode);
+static DList *brk_pt_list_delete (DList *head, int pc);
+static BreakPt *brk_pt_list_lookup (DList *head, int pc);
 static int brk_pt_cmp (AvrClass *d1, AvrClass *d2);
 
 /* Compare function for break points. */
@@ -133,7 +133,7 @@ brk_pt_cmp (AvrClass *d1, AvrClass *d2)
     return ((BreakPt *)d1)->pc - ((BreakPt *)d2)->pc;
 }
 
-static inline DList *
+static DList *
 brk_pt_list_add (DList *head, int pc, uint16_t opcode)
 {
     BreakPt *bp = brk_pt_new (pc, opcode);
@@ -141,7 +141,7 @@ brk_pt_list_add (DList *head, int pc, uint16_t opcode)
     return dlist_add (head, (AvrClass *)bp, brk_pt_cmp);
 }
 
-static inline DList *
+static DList *
 brk_pt_list_delete (DList *head, int pc)
 {
     BreakPt *bp = brk_pt_new (pc, 0);
@@ -152,7 +152,7 @@ brk_pt_list_delete (DList *head, int pc)
     return head;
 }
 
-static inline BreakPt *
+static BreakPt *
 brk_pt_list_lookup (DList *head, int pc)
 {
     BreakPt *found;
@@ -164,7 +164,7 @@ brk_pt_list_lookup (DList *head, int pc)
     return found;
 }
 
-static inline DList *
+static DList *
 brk_pt_iterator (DList *head, DListFP_Iter func, void *user_data)
 {
     return dlist_iterator (head, func, user_data);
@@ -192,13 +192,13 @@ struct _Irq
 
 #endif
 
-static inline Irq *irq_new (IntVect *vector, int state,
+static Irq *irq_new (IntVect *vector, int state,
                             unsigned int sleep_mode);
-static inline void irq_construct (Irq *irq, IntVect *vector, int state,
+static void irq_construct (Irq *irq, IntVect *vector, int state,
                                   unsigned int sleep_mode);
-static inline void irq_destroy (void *irq);
+static void irq_destroy (void *irq);
 
-static inline Irq *
+static Irq *
 irq_new (IntVect *vector, int state, unsigned int sleep_mode)
 {
     Irq *irq;
@@ -210,7 +210,7 @@ irq_new (IntVect *vector, int state, unsigned int sleep_mode)
     return irq;
 }
 
-static inline void
+static void
 irq_construct (Irq *irq, IntVect *vector, int state, unsigned int sleep_mode)
 {
     if (irq == NULL)
@@ -223,7 +223,7 @@ irq_construct (Irq *irq, IntVect *vector, int state, unsigned int sleep_mode)
     irq->sleep_mode = sleep_mode;
 }
 
-static inline void
+static void
 irq_destroy (void *irq)
 {
     Irq *_irq = (Irq *)irq;
@@ -234,10 +234,10 @@ irq_destroy (void *irq)
     class_destroy (irq);
 }
 
-static inline DList *irq_list_add (DList *head, IntVect *vector);
-static inline DList *irq_list_delete (DList *head, IntVect *vector);
+static DList *irq_list_add (DList *head, IntVect *vector);
+static DList *irq_list_delete (DList *head, IntVect *vector);
 #if 0                           /* TRoth/2002-09-15: This isn't used. ??? */
-static inline Irq *irq_list_lookup_addr (DList *head, IntVect *vector);
+static Irq *irq_list_lookup_addr (DList *head, IntVect *vector);
 #endif
 static int irq_cmp_addr (AvrClass *d1, AvrClass *d2);
 static int irq_cmp_pending (AvrClass *d1, AvrClass *d2);
@@ -250,7 +250,7 @@ irq_cmp_addr (AvrClass *d1, AvrClass *d2)
     return ((Irq *)d1)->vector->addr - ((Irq *)d2)->vector->addr;
 }
 
-static inline DList *
+static DList *
 irq_list_add (DList *head, IntVect *vector)
 {
     Irq *irq = irq_new (vector, 0, 0);
@@ -258,7 +258,7 @@ irq_list_add (DList *head, IntVect *vector)
     return dlist_add (head, (AvrClass *)irq, irq_cmp_addr);
 }
 
-static inline DList *
+static DList *
 irq_list_delete (DList *head, IntVect *vector)
 {
     Irq *irq = irq_new (vector, 0, 0);
@@ -270,7 +270,7 @@ irq_list_delete (DList *head, IntVect *vector)
 }
 
 #if 0                           /* TRoth/2002-09-15: This isn't used. ??? */
-static inline Irq *
+static Irq *
 irq_list_lookup_addr (DList *head, IntVect *vector)
 {
     Irq *found;
@@ -317,7 +317,7 @@ irq_cmp_pending (AvrClass *d1, AvrClass *d2)
    is in a sleep state, the can_wake mask could force the head of the list to
    not be the irq which gets vectored. */
 
-static inline IntVect *
+static IntVect *
 irq_get_pending_vector (DList *head, int state, unsigned int sleep_mode)
 {
     Irq *found;
@@ -330,7 +330,7 @@ irq_get_pending_vector (DList *head, int state, unsigned int sleep_mode)
 }
 
 #if 0                           /* TRoth/2002-09-15: This isn't used. ??? */
-static inline IntVect *
+static IntVect *
 irq_get_head_vector (DList *head)
 {
     return ((Irq *)dlist_get_head_data (head))->vector;
@@ -378,7 +378,9 @@ avr_core_construct (AvrCore *core, DevSuppDefn *dev)
     int PC_sz = dev_supp_get_PC_sz (dev);
     int stack_sz = dev_supp_get_stack_sz (dev);
     int sram_sz = dev_supp_get_sram_sz (dev);
+    int xram_sz = dev_supp_get_xram_sz (dev);
     int vtab_idx = dev_supp_get_vtab_idx (dev);
+    int xram_end, sram_end;
     int addr;
 
     if (core == NULL)
@@ -409,9 +411,17 @@ avr_core_construct (AvrCore *core, DevSuppDefn *dev)
 
     /* FIXME: hack to get it to compile. */
     if (dev_supp_has_ext_io_reg (dev))
-        core->mem = mem_new (0x1f, 0xff, 0xff + sram_sz, 0xffff);
+        sram_end = 0xff + sram_sz;
     else
-        core->mem = mem_new (0x1f, 0x5f, 0x5f + sram_sz, 0xffff);
+        sram_end = 0x5f + sram_sz;
+    if (xram_sz)
+        xram_end = 0xffff;
+    else
+        xram_end = sram_end;    /* i.e., no XRAM at all */
+    if (dev_supp_has_ext_io_reg (dev))
+        core->mem = mem_new (0x1f, 0xff, sram_end, xram_end);
+    else
+        core->mem = mem_new (0x1f, 0x5f, sram_end, xram_end);
 
     /* Attach the gpwr's to the memory bus. */
 
@@ -485,6 +495,35 @@ avr_core_construct (AvrCore *core, DevSuppDefn *dev)
         core->sram = NULL;
     }
 
+    if (xram_sz)
+    {
+        int base;
+        VDevice *xram;
+
+        if (dev_supp_has_ext_io_reg (dev))
+            base = SRAM_EXTENDED_IO_BASE + sram_sz;
+        else
+            base = SRAM_BASE + sram_sz;
+
+        core->xram = sram_new (base, xram_sz);
+        xram = (VDevice *)core->xram;
+
+        avr_message ("attach: External SRAM from 0x%04x to 0x%04x\n", base,
+                     (base + xram_sz - 1));
+
+        for (addr = base; addr < (base + xram_sz); addr++)
+        {
+            avr_core_attach_vdev (core, addr, "External SRAM", xram, 0, 0,
+                                  0xff, 0xff);
+        }
+    }
+    else
+    {
+        core->xram = NULL;
+    }
+
+    core->eeprom = NULL;
+
     /* Initialize the decoder lookup table */
 
     decode_init_lookup_table ();
@@ -576,10 +615,10 @@ extern inline int avr_core_get_sleep_mode (AvrCore *core);
 /*@{*/
 
 /** \brief Reads a word from flash memory. */
-static inline uint16_t avr_core_flash_read (AvrCore *core, int addr);
+static uint16_t avr_core_flash_read (AvrCore *core, int addr);
 
 /** \brief Writes a word to flash memory. */
-static inline void avr_core_flash_write (AvrCore *core, int addr,
+static void avr_core_flash_write (AvrCore *core, int addr,
                                          uint16_t val);
 
 /** \brief Writes a byte to flash memory.
@@ -588,7 +627,7 @@ static inline void avr_core_flash_write (AvrCore *core, int addr,
   * Use avr_core_flash_write() write to write a full word,
   * or avr_core_flash_write_hi8() to write the upper 8 bits.
   */
-static inline void avr_core_flash_write_lo8 (AvrCore *core, int addr,
+static void avr_core_flash_write_lo8 (AvrCore *core, int addr,
                                              uint8_t val);
 
 /** \brief Writes a byte to flash memory.
@@ -597,7 +636,7 @@ static inline void avr_core_flash_write_lo8 (AvrCore *core, int addr,
   * Use avr_core_flash_write() write to write a full word,
   * or avr_core_flash_write_lo8() to write the lower 8 bits.
   */
-static inline void avr_core_flash_write_hi8 (AvrCore *core, int addr,
+static void avr_core_flash_write_hi8 (AvrCore *core, int addr,
                                              uint8_t val);
 
 /*@}*/
@@ -610,13 +649,13 @@ static inline void avr_core_flash_write_hi8 (AvrCore *core, int addr,
   *
   * This accesses the \a register \a file and the \a SRAM.
   */
-static inline uint8_t avr_core_mem_read (AvrCore *core, int addr);
+static uint8_t avr_core_mem_read (AvrCore *core, int addr);
 
 /** \brief Writes a byte to memory.
   *
   * This accesses the \a register \a file and the \a SRAM.
   */
-static inline void avr_core_mem_write (AvrCore *core, int addr, uint8_t val);
+static void avr_core_mem_write (AvrCore *core, int addr, uint8_t val);
 
 /*@}*/
 
@@ -626,11 +665,11 @@ static inline void avr_core_mem_write (AvrCore *core, int addr, uint8_t val);
 
 /** \brief Get the value of the status register. */
 
-static inline uint8_t avr_core_sreg_get (AvrCore *core);
+static uint8_t avr_core_sreg_get (AvrCore *core);
 
 /** \brief Set the value of the status register. */
 
-static inline void avr_core_sreg_set (AvrCore *core, uint8_t v);
+static void avr_core_sreg_set (AvrCore *core, uint8_t v);
 
 /** \brief Get the value of bit \c b of the status register. */
 
@@ -663,10 +702,10 @@ extern inline void avr_core_rampz_set (AvrCore *core, uint8_t v);
 /*@{*/
 
 /** \brief Returns a GPWR's(\a r0-r31) value. */
-static inline uint8_t avr_core_gpwr_get (AvrCore *core, int reg);
+static uint8_t avr_core_gpwr_get (AvrCore *core, int reg);
 
 /** \brief Writes a GPWR's (\a r0-r31) value. */
-static inline void avr_core_gpwr_set (AvrCore *core, int reg, uint8_t val);
+static void avr_core_gpwr_set (AvrCore *core, int reg, uint8_t val);
 
 /*@}*/
 
@@ -707,7 +746,7 @@ extern inline void avr_core_io_write (AvrCore *core, int reg, uint8_t val);
 
 /** \brief Read an io register into val and put the name of the register into
   * buf. */
-static inline void avr_core_io_fetch (AvrCore *core, int reg, uint8_t * val,
+static void avr_core_io_fetch (AvrCore *core, int reg, uint8_t * val,
                                       char *buf, int bufsiz);
 
 /*@}*/
@@ -746,17 +785,17 @@ extern inline int32_t avr_core_PC_size (AvrCore *core);
  *
  * This is flash_size / 2.
  */
-static inline int32_t avr_core_PC_max (AvrCore *core);
+static int32_t avr_core_PC_max (AvrCore *core);
 
 /** \brief Return the current of the Program Counter. */
-static inline int32_t avr_core_PC_get (AvrCore *core);
+static int32_t avr_core_PC_get (AvrCore *core);
 
 /** \brief Set the Program Counter to val.
  *
  * If val is not in the valid range of PC values, it is adjusted to fall in
  * the valid range.
  */
-static inline void avr_core_PC_set (AvrCore *core, int32_t val);
+static void avr_core_PC_set (AvrCore *core, int32_t val);
 
 /** \brief Increment the Program Counter by val.
  *
