@@ -1,5 +1,5 @@
 /*
- * $Id: disp.c,v 1.18 2003/12/01 05:48:34 troth Exp $
+ * $Id: disp.c,v 1.19 2008/01/07 16:58:04 joerg_wunsch Exp $
  *
  ****************************************************************************
  *
@@ -380,7 +380,7 @@ disp_print_io_reg_name (int reg, char *long_name)
 
     strncpy (io_reg_names[reg], long_name, MAX_IO_REG_BUF);
     strncpy (name, long_name, wn);
-    mvwprintw (win_io_reg, r, c, "%-*s", wn, name);
+    mvwprintw (win_io_reg, r, c, "%-*.*s", wn, wn, name);
 }
 
 /* Print the IO Registers */
@@ -1059,7 +1059,6 @@ main (int argc, char **argv)
     {
         io_reg_names[i] =
             (char *)malloc ((MAX_IO_REG_BUF + 1) * sizeof (char));
-        memset (io_reg_names[i], '\0', MAX_IO_REG_BUF + 1);
         strncpy (io_reg_names[i], "Reserved", MAX_IO_REG_BUF);
     }
 
