@@ -1,5 +1,5 @@
 /*
- * $Id: intvects.c,v 1.17 2004/02/26 07:22:15 troth Exp $
+ * $Id: intvects.c,v 1.18 2008/01/06 21:34:16 joerg_wunsch Exp $
  *
  ****************************************************************************
  *
@@ -219,6 +219,56 @@ static IntVectTable vtab_atmega16 = {
                         0x28, 0x00, { 0x57, 1<<7 }, NO_BIT }
 };
 
+/*
+ * Vector Table for devices:
+ *   atmega32
+ */
+
+static IntVectTable vtab_atmega32 = {
+    .RESET          = { "IRQ_RESET",
+                        0x00, 0x00, NO_BIT, NO_BIT },
+    .INT0           = { "IRQ_INT0",
+                        0x02, 0x00, { 0x5b, 1<<6 }, { 0x5a, 1<<6 } },
+    .INT1           = { "IRQ_INT1",
+                        0x04, 0x00, { 0x5b, 1<<7 }, { 0x5a, 1<<7 } },
+    .TIMER2_COMP    = { "IRQ_TIMER2_COMP",
+                        0x06, 0x00, { 0x59, 1<<7 }, { 0x58, 1<<7 } },
+    .TIMER2_OVF     = { "IRQ_TIMER2_OVF",
+                        0x08, 0x00, { 0x59, 1<<6 }, { 0x58, 1<<6 } },
+    .TIMER1_CAPT    = { "IRQ_TIMER1_CAPT",
+                        0x0a, 0x00, { 0x59, 1<<5 }, { 0x58, 1<<5 } },
+    .TIMER1_COMPA   = { "IRQ_TIMER1_COMPA",
+                        0x0c, 0x00, { 0x59, 1<<4 }, { 0x58, 1<<4 } },
+    .TIMER1_COMPB   = { "IRQ_TIMER1_COMPB",
+                        0x0e, 0x00, { 0x59, 1<<3 }, { 0x58, 1<<3 } },
+    .TIMER1_OVF     = { "IRQ_TIMER1_OVF",
+                        0x10, 0x00, { 0x59, 1<<2 }, { 0x58, 1<<2 } },
+    .TIMER0_OVF     = { "IRQ_TIMER0_OVF",
+                        0x12, 0x00, { 0x59, 1<<0 }, { 0x58, 1<<0 } },
+    .SPI_STC        = { "IRQ_SPI_STC",
+                        0x14, 0x00, { 0x2d, 1<<7 }, NO_BIT },
+    .UART_RX        = { "IRQ_UART_RX",
+                        0x16, 0x00, { 0x2a, 1<<7 }, { 0x2b, 1<<7 } },
+    .UART_UDRE      = { "IRQ_UART_UDRE",
+                        0x18, 0x00, { 0x2a, 1<<5 }, { 0x2b, 1<<5 } },
+    .UART_TX        = { "IRQ_UART_TX",
+                        0x1a, 0x00, { 0x2a, 1<<6 }, { 0x2b, 1<<6 } },
+    .ADC            = { "IRQ_ADC",
+                        0x1c, 0x00, { 0x26, 1<<3 }, { 0x26, 1<<4 } },
+    .EE_READY       = { "IRQ_EE_READY",
+                        0x1e, 0x00, { 0x3c, 1<<3 }, NO_BIT },
+    .ANA_COMP       = { "IRQ_ANA_COMP",
+                        0x20, 0x00, { 0x28, 1<<3 }, { 0x28, 1<<4 } },
+    .TWI            = { "IRQ_TWI",
+                        0x22, 0x00, { 0x57, 1<<0 }, { 0x56, 1<<7 } },
+    .INT2           = { "IRQ_INT2",
+                        0x24, 0x00, { 0x5b, 1<<5 }, { 0x5a, 1<<5 } },
+    .TIMER0_COMP    = { "IRQ_TIMER0_COMP",
+                        0x26, 0x00, { 0x59, 1<<1 }, { 0x58, 1<<1 } },
+    .SPM_READY      = { "IRQ_SPM_READY",
+                        0x28, 0x00, { 0x57, 1<<7 }, NO_BIT }
+};
+
 
 /*
  * Vector Table for devices:
@@ -468,6 +518,7 @@ IntVectTable *global_vtable_list[] = {
     &vtab_at90s4414,
     &vtab_atmega8,
     &vtab_atmega16,
+    &vtab_atmega32,
     &vtab_atmega103,
     &vtab_atmega128,
     &vtab_at43usb355,
